# Copyright 2021 Trevor Lauder.
# SPDX-License-Identifier: MIT

import datetime
import logging


class LogHandler:
    def __init__(self):
        logging.basicConfig(format="%(levelname)s:     %(message)s", level=logging.INFO)

        self.logging = logging

    def log(self, message: str = None):
        if message:
            timestamp = str(datetime.datetime.now().strftime("%d %b %Y %H:%M:%S"))

            self.logging.info("{}: {}".format(timestamp, message))



# Copyright 2021 Trevor Lauder.
# SPDX-License-Identifier: MIT

import json

from redis import Redis


class DataStore:
    def __init__(self, config: dict, host: str):
        self.config = config

        self.r = Redis(host=host, decode_responses=True)

    def publish_command(self, name: str, channel: str = "controller"):
        self.r.publish(
            channel,
            json.dumps({"name": name, "command": self.config["commands"][name]}),
        )



# Copyright 2021 Trevor Lauder.
# SPDX-License-Identifier: MIT

from redis import Redis


class DataStore:
    def __init__(self, host: str):
        self.r = Redis(host=host, decode_responses=True)

    def exists(self, key: str):
        return self.r.exists(key) != 0

    def get(self, key: str):
        return self.r.get(key)



# Copyright 2021 Trevor Lauder.
# SPDX-License-Identifier: MIT

import re
import socket
import sys


class BedSocket:
    def __init__(
        self,
        config: dict,
        datastore: object,
        bed_logging: object,
        host: str = "0.0.0.0",
        port: int = 23778,
    ):
        self.s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.s.bind((host, port))
        self.s.listen(1)

        self.config = config
        self.datastore = datastore
        self.bed_logging = bed_logging

        self.address = None
        self.connection = None

    def receive_data(self):
        self.connection.settimeout(2.0)

        while True:
            socket_data = None

            try:
                socket_data = self.connection.recv(2048)
            except OSError:
                next
            except socket.timeout:
                continue

            if socket_data is not None and len(socket_data) == 0:
                self.connection.close()

                self.bed_logging.log(
                    message="Connection Closed: {}:{}".format(
                        self.address[0], self.address[1]
                    )
                )

                sys.exit(1)
            elif socket_data is not None:
                break

        return socket_data

    def socket_send_data(self, data: str):
        try:
            self.connection.send(bytes.fromhex(data))
        except OSError:
            self.connection.close()

            self.bed_logging.log(
                message="Socket Closed While Trying to Send: {}:{} ()".format(
                    self.address[0], self.address[1]
                )
            )

            sys.exit(1)

    def accept_connection(self):
        self.bed_logging.log(message="Started and Waiting for a New Connection!")

        self.connection, self.address = self.s.accept()

        self.bed_logging.log(
            message="Connection Opened: {}:{}".format(self.address[0], self.address[1])
        )

    def api(self, online: bool, loop: bool = True):
        self.online = online

        do_loop = True

        while do_loop:
            socket_data = self.receive_data()

            socket_data_hex = socket_data.hex()

            self.bed_logging.log(
                message="Received from {}:{}: {}".format(
                    self.address[0],
                    self.address[1],
                    socket_data_hex,
                )
            )

            for response in self.config["responses"]:
                socket_data_match = re.search(response, socket_data_hex)

                if socket_data_match:
                    respond_data = self.config["responses"][response]

                    if "groups" in respond_data and isinstance(
                        respond_data["groups"], list
                    ):
                        for count, group in enumerate(respond_data["groups"], start=1):
                            group_value = bytes.fromhex(
                                socket_data_match.group(count)
                            ).decode()

                            self.datastore.update(
                                group=group,
                                group_value=group_value,
                            )

                    self.socket_send_data(data=respond_data["response"])

                    self.bed_logging.log(
                        message="Sent to {}:{} in Response: {}".format(
                            self.address[0],
                            self.address[1],
                            respond_data["response"],
                        )
                    )

                    if not self.online:
                        command = self.config["commands"]["online"]["send"]

                        self.socket_send_data(data=command)

                        self.online = True

                        self.bed_logging.log(
                            message="Sent to {}:{} (to put online): {}".format(
                                self.address[0], self.address[1], command
                            )
                        )

            do_loop = loop



# Copyright 2021 Trevor Lauder.
# SPDX-License-Identifier: MIT

import os

import uvicorn  # type: ignore

from bn_adjustable_bed.controller_api.api import Api
from bn_adjustable_bed.controller_api.config import get_config
from bn_adjustable_bed.controller_api.datastore import DataStore


def main():
    config = get_config()

    datastore = DataStore(config=config, host=os.environ["REDIS_HOST"])

    api = Api(config=config, datastore=datastore)

    uvicorn.run(api, host="0.0.0.0", port=80)



# Copyright 2021 Trevor Lauder.
# SPDX-License-Identifier: MIT

import os
import threading

from bn_adjustable_bed.bed_socket.bedsocket import BedSocket
from bn_adjustable_bed.bed_socket.config import get_config
from bn_adjustable_bed.bed_socket.datastore import DataStore
from bn_adjustable_bed.bed_socket.loghandler import LogHandler
from bn_adjustable_bed.bed_socket.process_commands import process_commands


def main():
    config = get_config()

    bed_logging = LogHandler()

    datastore = DataStore(bed_logging=bed_logging, host=os.environ["REDIS_HOST"])

    bed_socket = BedSocket(config=config, datastore=datastore, bed_logging=bed_logging)

    online = False

    while True:
        bed_socket.accept_connection()

        process_commands_kwargs = {
            "bed_logging": bed_logging,
            "datastore": datastore,
            "bed_socket": bed_socket,
        }

        bed_socket_api_kwargs = {"online": online}

        process_commands_thread = threading.Thread(
            target=process_commands, kwargs=process_commands_kwargs
        )
        process_commands_thread.start()

        bed_socket_api_thread = threading.Thread(
            target=bed_socket.api, kwargs=bed_socket_api_kwargs
        )
        bed_socket_api_thread.start()

        online = True
